import java.sql.*;

import java.io.Console;

import java.io.FileInputStream;
import java.io.FileOutputStream;

import java.io.FileNotFoundException;
import java.io.IOException;

public class BlobWriting {
	private static void loadDriver() {
		try {
			// this will load the MySQL driver
			// make sure the corresponding jar file is included on the java
			// CLASSPATH
			Class.forName("com.mysql.jdbc.Driver");
			System.out.println("Driver instance ok...");
		} catch (Exception e) {
			System.err.println("Unable to load driver.");
			e.printStackTrace();
		}
	}

	// Establish a connection with specified database. Return connection object
	private static Connection establish_connection(String database_name, String sql_username, String sql_passwd) {
		Connection conn = null;
		try {

			System.out.println("Establishing connection with satoshi..");
			conn = DriverManager.getConnection(//"jdbc:mysql://satoshi.cis.uncw.edu/" + 
            "jdbc:mysql://152.20.12.152:3306/" + 
            database_name
					+ "?noAccessToProcedureBodies=true"+"&useSSL=false" + "&user=" + sql_username + "&password=" + sql_passwd);

			System.out.println("Connection with satoshi.cis.uncw.edu established.");
		} catch (SQLException e) {
			System.out.println("SQLException: " + e.getMessage());
			System.out.println("SQLState:     " + e.getSQLState());
			System.out.println("VendorError:  " + e.getErrorCode());
            e.printStackTrace();
		}

		return conn;
	}

	private static void storeImage(Connection conn) {
		try {
			Console c = System.console();

			System.out.println("Store image demo...");

			System.out.println("Enter id for image to be stored in database: ");
			String id = c.readLine();
			int imageId = Integer.parseInt(id);

			System.out.println("Enter image file name: ");
			String filename = c.readLine();

			FileInputStream iStream = null;

			try {
				iStream = new FileInputStream(filename);

			} catch (FileNotFoundException fnf) {
				fnf.printStackTrace();
			}

			PreparedStatement pStmt = conn
					.prepareStatement("insert into imageTable (imageId, imageData) values('" + imageId + "', ?)");

			// Assign stream to prepared statement placeholder
			pStmt.setBlob(1, iStream);
			System.out.println("Prepared statement is " + pStmt);

			// Execute update
			pStmt.executeUpdate();

			// check database using query
			// select imageId, length(imageData) from imageTable;
			// length should be same as size of image in bytes
			// Note: different blob types have different max sizes
			// TINYBLOB: maximum length of 255 bytes
			// BLOB: maximum length of 65,535 bytes
			// MEDIUMBLOB: maximum length of 16,777,215 bytes
			// LONGBLOB: maximum length of 4,294,967,295 bytes
		} catch (SQLException e) {
			System.out.println("SQLException: " + e.getMessage());
			System.out.println("SQLState:     " + e.getSQLState());
			System.out.println("VendorError:  " + e.getErrorCode());
		}
	}

	private static void readImage(Connection conn) {
		try {
			
			Blob image = null;
			Statement stmt = conn.createStatement();
            
            Console c = System.console();

			System.out.println("Retrieve image demo...");

			System.out.println("Enter id of image to be retrieved: ");
			String id = c.readLine();
			int imageId = Integer.parseInt(id);
            
			PreparedStatement pStmt = conn
					.prepareStatement("select imageData from imageTable where imageId = ?");
                    
            //Assign imageId to prepared statement placeholder 
            pStmt.setInt(1, imageId);
            
			System.out.println("Prepared statement is " + pStmt);

			// Execute update
			ResultSet result = pStmt.executeQuery();
			
			
			while (result.next()) {
                image = result.getBlob("imageData");
                int length = (int ) image.length();
                
                byte [] data = image.getBytes(1, length);
                
			System.out.println("Enter file name to store retrieved image: ");
			String filename = c.readLine();
            
			try {
				FileOutputStream fos = new FileOutputStream(filename);
				fos.write(data);
				fos.close();

			} catch (FileNotFoundException fnf) {
				fnf.printStackTrace();
			}
			catch (IOException ioe) {
				ioe.printStackTrace();
			}
			}
			
		} catch (SQLException e) {
			System.out.println("SQLException: " + e.getMessage());
			System.out.println("SQLState:     " + e.getSQLState());
			System.out.println("VendorError:  " + e.getErrorCode());
		}
	}

	public static void main(String[] args) {
        
        if (args.length == 1) {
            loadDriver();

            Console c = System.console();
            // read user name, database name, and password
            String database_name = c.readLine("Database name? ");
            String username = c.readLine("User Name? ");
            char[] password = c.readPassword("Password? ");

            Connection conn = establish_connection(database_name, username, new String(password));
            
            if (args[0].equals("read"))
                readImage(conn);
            else if (args[0].equals("store"))
                storeImage(conn);
        }
        else {
            System.out.println("Usage: java BlobWriting read|store");
        }
	}
}
